/*-----------------------------------------------------------------------
 *     Copyright (C) 2000-2004 GFD Dennou Club. All rights reserved.
 *----------------------------------------------------------------------*/

#include <stdlib.h>

static float **txs, **tys;

/*--- PRIVATE ---*/
static float **dcl_alloc_2d(n, m)  
     int n, m;
{
    /* Allocate a 2D array of shape [n][m].
     * Use dcl_free_2d to free the array */
    float **b;
    int i;
    b = (float **)malloc((size_t) n*sizeof(float*));
    b[0] = (float *)malloc((size_t) n*m*sizeof(float));
    for(i=1;i<n;i++) {b[i] = b[i-1]+m;}
    return(b);
}
static void dcl_free_2d(b)
    double **b;
{
    free(b[0]);
    free(b);
}

/*--- PUBLIC ---*/
void dcl_stgitp_(i, j, tx11, tx21, tx12, tx22, ty11, ty21, ty12, ty22)
     int *i, *j;
     float *tx11, *tx21, *tx12, *tx22;
     float *ty11, *ty21, *ty12, *ty22;
{
    /* NOTE: C indices are 1 smaller than Fortran indices */
    *tx11 = txs[*j-1][*i-1];
    *tx21 = txs[*j-1][*i];
    *tx12 = txs[*j][*i-1];
    *tx22 = txs[*j][*i];
    *ty11 = tys[*j-1][*i-1];
    *ty21 = tys[*j-1][*i];
    *ty12 = tys[*j][*i-1];
    *ty22 = tys[*j][*i];
}

void dcl_stsitp_(rundef, iuxinc, iuyinc, nx, ny, tx, ty, uxs, uys)
     float *rundef;
     int *iuxinc, *iuyinc, *nx, *ny;
     float *tx, *ty, *uxs, *uys;
{
    static int first = 1;
    int i,j;
    int txundef, tyundef;

    /* free if not first time */
    if(!first){
	dcl_free_2d(txs);
	dcl_free_2d(tys);
    }

    /* store TX and TY */

    txs = dcl_alloc_2d(*ny, *nx);
    tys = dcl_alloc_2d(*ny, *nx);
    first = 0;

    if (tx[0]==(*rundef)) {
	/* Assume tx[j][i] == ux[i] */
	txundef = 1;
	for(j=0;j<*ny;j++){
	    for(i=0;i<*nx;i++){
		txs[j][i] = uxs[i];
	    }
	}
    } else {
	txundef = 0;
    }

    if (ty[0]==(*rundef)) {
	/* Assume ty[j][i] == uy[j] */
	tyundef = 1;
	for(j=0;j<*ny;j++){
	    for(i=0;i<*ny;i++){
		tys[j][i] = uys[j];
	    }
	}
    } else {
	tyundef = 0;
    }

    if((*iuxinc) && (*iuyinc) ){
	for(j=0;j<*ny;j++){
	    for(i=0;i<*nx;i++){
		if(!txundef) txs[j][i] = tx[*nx*j+i];
		if(!tyundef) tys[j][i] = ty[*nx*j+i];
	    }
	}
    } else if ( !(*iuxinc) && (*iuyinc) ) {
	for(j=0;j<*ny;j++){
	    for(i=0;i<*nx;i++){
		if(!txundef) txs[j][*nx-1-i] = tx[*nx*j+i];
		if(!tyundef) tys[j][*nx-1-i] = ty[*nx*j+i];
	    }
	}
    } else if ( (*iuxinc) && !(*iuyinc) ) {
	for(j=0;j<*ny;j++){
	    for(i=0;i<*nx;i++){
		if(!txundef) txs[*ny-1-j][i] = tx[*nx*j+i];
		if(!tyundef) tys[*ny-1-j][i] = ty[*nx*j+i];
	    }
	}
    } else {
	for(j=0;j<*ny;j++){
	    for(i=0;i<*nx;i++){
		if(!txundef) txs[*ny-1-j][*nx-1-i] = tx[*nx*j+i];
		if(!tyundef) tys[*ny-1-j][*nx-1-i] = ty[*nx*j+i];
	    }
	}
    }

}
